/*******************************************************************************
 * Fraco - Eclipse plug-in to detect fragile comments
 *
 * Copyright (C) 2019 McGill University
 *     
 * Eclipse Public License - v 2.0
 *
 *  THE ACCOMPANYING PROGRAM IS PROVIDED UNDER THE TERMS OF THIS ECLIPSE
 *  PUBLIC LICENSE v2.0. ANY USE, REPRODUCTION OR DISTRIBUTION
 *  OF THE PROGRAM CONSTITUTES RECIPIENT'S ACCEPTANCE OF THIS AGREEMENT.
 *******************************************************************************/
/**
 * 
 */
package ca.mcgill.cs.stg.fraco.Properties;

import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.search.IJavaSearchConstants;
import org.eclipse.jdt.core.search.IJavaSearchScope;
import org.eclipse.jdt.core.search.SearchEngine;
import org.eclipse.jdt.core.search.SearchMatch;
import org.eclipse.jdt.core.search.SearchParticipant;
import org.eclipse.jdt.core.search.SearchPattern;
import org.eclipse.jdt.core.search.SearchRequestor;

import ca.mcgill.cs.stg.fraco.AstBuilder;
import ca.mcgill.cs.stg.fraco.utils.Identifier;

/**
 * @author inder
 *
 */
public class IdentifierPropertiesResolver {

	/**
	 * Creates an instance of class Identifier() and sets the required
	 * properties like class name and package name.
	 */
	public Identifier setCurrentIdentifier() {

		if (AstBuilder._renamedElement != null) {
			// searchRootElement();
			AstBuilder._identifier = new Identifier();
			AstBuilder._identifier.set_name(AstBuilder._renamedElement.getElementName().toString());
			ICompilationUnit classFile = (ICompilationUnit) AstBuilder._renamedElement
					.getAncestor(IJavaElement.COMPILATION_UNIT);
			AstBuilder._identifier.set_cuName(classFile.getElementName());

			// sets the name if package which contains the identifier.
			setIdentifierPackageName(AstBuilder._renamedElement);

			// if the identifier is of type local variable, will have ancestor
			// of type method else ancestor type will be TYPE.
			if (AstBuilder._renamedElement.getAncestor(IJavaElement.METHOD) != null) {
				setIdentifierMethodandParentName();
			} else if (AstBuilder._renamedElement.getAncestor(IJavaElement.TYPE) != null) {
				IType type = (IType) AstBuilder._renamedElement.getAncestor(IJavaElement.TYPE);
				if (type.getElementName() != null) {
					AstBuilder._identifier.set_className(type.getElementName());
				} else if (type.getDeclaringType() != null && type.getDeclaringType().getElementName() != null)
					AstBuilder._identifier.set_className(type.getDeclaringType().getElementName());
				// sets the parent class name of the ancestor class of
				// identifier
				setIdentifierParentClassName(type);
			}
		}
		return AstBuilder._identifier;
	}

	private void searchRootElement() {
		try {
			if (AstBuilder._renamedElement instanceof IMethod) {

				IJavaSearchScope scope = SearchEngine
						.createJavaSearchScope(new IJavaElement[] { AstBuilder._renamedElement.getJavaProject() });
				SearchPattern searchParttern = SearchPattern.createPattern((IMethod) AstBuilder._renamedElement,
						IJavaSearchConstants.DECLARATIONS);
				SearchRequestor requestor = new SearchRequestor() {
					@Override
					public void acceptSearchMatch(SearchMatch match) {
						System.out.println(match.getElement());
					}
				};
				SearchEngine searchEngine = new SearchEngine();
				searchEngine.search(searchParttern,
						new SearchParticipant[] { SearchEngine.getDefaultSearchParticipant() }, scope, requestor,
						new NullProgressMonitor());
			}
		} catch (Exception e) {

		}
	}

	/**
	 * Sets the parent class name of the ancestor class of identifier. Needed
	 * for scope anaylsis of identifier and comments
	 * 
	 * @param type
	 *            is the parent class of identifier
	 */
	private void setIdentifierParentClassName(IType type) {
		if (type.getParent().getElementType() == IJavaElement.TYPE)
			AstBuilder._identifier.set_parentClassName(type.getParent().getElementName());
		else
			AstBuilder._identifier.set_parentClassName(type.getCompilationUnit().getElementName());
	}

	/**
	 * Recursively finds and sets the name of the package which contains the
	 * identifier.
	 * 
	 * @param element
	 *            is the renamed java element
	 */
	private void setIdentifierPackageName(IJavaElement element) {
		if (element.getAncestor(IJavaElement.PACKAGE_FRAGMENT) == null) {
			setIdentifierPackageName(element.getParent());
		} else {
			IPackageFragment pac = (IPackageFragment) element.getAncestor(IJavaElement.PACKAGE_FRAGMENT);
			AstBuilder._identifier.set_packageName(pac.getElementName());
		}
	}

	/**
	 * if the renamed element is a LOCAL or FORMAL PARAMETER, it finds and sets
	 * the method name and containing class name of the identifier.
	 */
	private void setIdentifierMethodandParentName() {
		IMethod method = (IMethod) AstBuilder._renamedElement.getAncestor(IJavaElement.METHOD);
		AstBuilder._identifier.set_methodName(method.getElementName());

		if (method.getParent() != null && method.getParent().getElementType() == IJavaElement.TYPE) {
			IType type = (IType) method.getParent();
			if (type.getElementName() != null)
				AstBuilder._identifier.set_className(type.getElementName());
			else if (type.getDeclaringType() != null && type.getDeclaringType().getElementName() != null)
				AstBuilder._identifier.set_className(type.getDeclaringType().getElementName());
		}
	}
}
