/*******************************************************************************
 * Fraco - Eclipse plug-in to detect fragile comments
 *
 * Copyright (C) 2019 McGill University
 *     
 * Eclipse Public License - v 2.0
 *
 *  THE ACCOMPANYING PROGRAM IS PROVIDED UNDER THE TERMS OF THIS ECLIPSE
 *  PUBLIC LICENSE v2.0. ANY USE, REPRODUCTION OR DISTRIBUTION
 *  OF THE PROGRAM CONSTITUTES RECIPIENT'S ACCEPTANCE OF THIS AGREEMENT.
 *******************************************************************************/
/**
 * 
 */
package ca.mcgill.cs.stg.fraco.Properties.handlers;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.CompilationUnit;
import org.eclipse.jdt.core.dom.rewrite.ASTRewrite;
import org.eclipse.jdt.core.search.SearchPattern;
import org.eclipse.jdt.internal.ui.text.correction.AssistContext;
import org.eclipse.jdt.internal.ui.text.java.AbstractJavaCompletionProposal;
import org.eclipse.jdt.ui.actions.AddGetterSetterAction;
import org.eclipse.jdt.ui.text.java.IInvocationContext;
import org.eclipse.jdt.ui.text.java.IJavaCompletionProposal;
import org.eclipse.jdt.ui.text.java.IProblemLocation;
import org.eclipse.jdt.ui.text.java.IQuickFixProcessor;
import org.eclipse.jdt.ui.text.java.correction.ASTRewriteCorrectionProposal;
import org.eclipse.jface.viewers.StyledString;
import org.eclipse.ui.IWorkbenchSite;

import ca.mcgill.cs.stg.fraco.AstBuilder;
import ca.mcgill.cs.stg.fraco.DetectFragileComments;

/**
 * @author inder
 *
 */
public class ProblemIdQuickFix implements IQuickFixProcessor {

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jdt.ui.text.java.IQuickFixProcessor#hasCorrections(org.
	 * eclipse.jdt.core.ICompilationUnit, int)
	 */
	@Override
	public boolean hasCorrections(ICompilationUnit unit, int problemId) {
		// TODO Auto-generated method stub
		if (DetectFragileComments._markerIds != null) {
			for (IMarker marker : DetectFragileComments._markerIds) {
				if (marker.getId() == problemId)
					return true;
			}
		}
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jdt.ui.text.java.IQuickFixProcessor#getCorrections(org.
	 * eclipse.jdt.ui.text.java.IInvocationContext,
	 * org.eclipse.jdt.ui.text.java.IProblemLocation[])
	 */
	@Override
	public IJavaCompletionProposal[] getCorrections(IInvocationContext context, IProblemLocation[] locations)
			throws CoreException {

		return new IJavaCompletionProposal[] { new AbstractJavaCompletionProposal() {
			public org.eclipse.jface.viewers.StyledString getStyledDisplayString() {
				ICompilationUnit compilationUnit = context.getCompilationUnit();
				String newName = compilationUnit.findPrimaryType().getElementName();
				return new StyledString(compilationUnit.findPrimaryType().getElementName());
			}

			protected int getPatternMatchRule(String pattern, String string) {
				return SearchPattern.R_EXACT_MATCH | SearchPattern.R_CASE_SENSITIVE;
			};

			public void apply(org.eclipse.jface.text.ITextViewer viewer, char trigger, int stateMask, int offset) {

				/*
				 * if(context instanceof AssistContext) { AssistContext
				 * assistContext = (AssistContext) context;
				 * AddGetterSetterAction addGetterSetterAction = new
				 * AddGetterSetterAction((IWorkbenchSite)assistContext.getEditor
				 * ());
				 * 
				 * addGetterSetterAction.run(); }
				 */
				ICompilationUnit compilationUnit = context.getCompilationUnit();
				CompilationUnit unit = AstBuilder.parse(compilationUnit);

				ASTRewrite rewrite = ASTRewrite.create(unit.getAST());
				ASTRewriteCorrectionProposal astrewrite = new ASTRewriteCorrectionProposal(
						"Replace the fragile instance with new name", compilationUnit, rewrite, 10);

			}

		} };

	}

}
