/*******************************************************************************
 * Fraco - Eclipse plug-in to detect fragile comments
 *
 * Copyright (C) 2019 McGill University
 *     
 * Eclipse Public License - v 2.0
 *
 *  THE ACCOMPANYING PROGRAM IS PROVIDED UNDER THE TERMS OF THIS ECLIPSE
 *  PUBLIC LICENSE v2.0. ANY USE, REPRODUCTION OR DISTRIBUTION
 *  OF THE PROGRAM CONSTITUTES RECIPIENT'S ACCEPTANCE OF THIS AGREEMENT.
 *******************************************************************************/
package ca.mcgill.cs.stg.fraco.visitors;

import java.util.ArrayList;
import java.util.List;
import org.eclipse.jdt.core.dom.ASTVisitor;
import org.eclipse.jdt.core.dom.ConstructorInvocation;
import org.eclipse.jdt.core.dom.MethodDeclaration;
import org.eclipse.jdt.core.dom.MethodInvocation;
import org.eclipse.jdt.core.dom.MethodRef;
import org.eclipse.jdt.core.dom.MethodRefParameter;

/**
 * MethodVisitor is used traverse all the nodes related to methods in an AST.
 * 
 * @author inder
 *
 */
public class MethodVisitor extends ASTVisitor {

	List<MethodDeclaration> methodDeclarations = new ArrayList<MethodDeclaration>();
	List<MethodInvocation> methodInvocations = new ArrayList<MethodInvocation>();
	List<MethodRef> methodRefs = new ArrayList<MethodRef>();
	List<MethodRefParameter> methodRefParameters = new ArrayList<MethodRefParameter>();
	List<ConstructorInvocation> constructors = new ArrayList<ConstructorInvocation>();

	/*
	 * (non-Javadoc) Traverses all the method declaration nodes.
	 * 
	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.
	 * MethodDeclaration)
	 */
	@Override
	public boolean visit(MethodDeclaration node) {

		methodDeclarations.add(node);
		return super.visit(node);
	}

	/**
	 * @return the list of method declarations in an AST.
	 */
	public List<MethodDeclaration> getMethodDeclarations() {
		return methodDeclarations;
	}

	/*
	 * (non-Javadoc) Traverses the method invocation nodes in an AST
	 * 
	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.
	 * MethodInvocation)
	 */
	@Override
	public boolean visit(MethodInvocation node) {

		methodInvocations.add(node);
		return super.visit(node);
	}

	/**
	 * @return the list of method invocations in an AST.
	 */
	public List<MethodInvocation> getMethodInvocations() {
		return methodInvocations;
	}

	/*
	 * (non-Javadoc) traverses the method reference nodes in an AST.
	 * 
	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.
	 * MethodRef)
	 */
	@Override
	public boolean visit(MethodRef node) {
		methodRefs.add(node);
		return super.visit(node);
	}

	/**
	 * @return the list of method reference nodes in an AST
	 */
	public List<MethodRef> getMethodRefs() {
		return methodRefs;
	}

	/*
	 * @Override public boolean visit(MethodRefParameter node) {
	 * methodRefParameters.add(node); return super.visit(node); }
	 * 
	 * public List<MethodRefParameter> getMethodRefParameters() { return
	 * methodRefParameters; }
	 */

	/*
	 * (non-Javadoc) traverses the constructor invocation nodes in an AST.
	 * 
	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.
	 * ConstructorInvocation)
	 */
	@Override
	public boolean visit(ConstructorInvocation node) {
		constructors.add(node);
		return super.visit(node);
	}

	/**
	 * @return the list of constructor invocation nodes.
	 */
	public List<ConstructorInvocation> getConstructors() {
		return constructors;
	}

}
