/*******************************************************************************
 * Fraco - Eclipse plug-in to detect fragile comments
 *
 * Copyright (C) 2019 McGill University
 *     
 * Eclipse Public License - v 2.0
 *
 *  THE ACCOMPANYING PROGRAM IS PROVIDED UNDER THE TERMS OF THIS ECLIPSE
 *  PUBLIC LICENSE v2.0. ANY USE, REPRODUCTION OR DISTRIBUTION
 *  OF THE PROGRAM CONSTITUTES RECIPIENT'S ACCEPTANCE OF THIS AGREEMENT.
 *******************************************************************************/
/**
 * 
 */
package ca.mcgill.cs.stg.fraco.visitors;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jdt.core.dom.ASTVisitor;
import org.eclipse.jdt.core.dom.DoStatement;
import org.eclipse.jdt.core.dom.Expression;

import org.eclipse.jdt.core.dom.ForStatement;
import org.eclipse.jdt.core.dom.IfStatement;

import org.eclipse.jdt.core.dom.WhileStatement;
import org.eclipse.jdt.core.dom.Statement;

/**
 * StatementsVisitor is used to visit expressions an statements node in AST. We
 * are only travering those statements where a local variable or field might be
 * used.
 * 
 * @author inderjot
 *
 */
public class StatementsVisitor extends ASTVisitor {

	List<Statement> statements = new ArrayList<Statement>();
	List<Expression> expressions = new ArrayList<Expression>();

	/*
	 * (non-Javadoc) Traverses all the Do statement nodes.
	 * 
	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.
	 * DoStatement)
	 */
	@Override
	public boolean visit(DoStatement node) {

		expressions.add(node.getExpression());
		statements.add(node);
		return super.visit(node);
	}

	/*
	 * (non-Javadoc) Traverses all the If statement nodes.
	 * 
	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.
	 * IfStatement)
	 */
	@Override
	public boolean visit(IfStatement node) {
		expressions.add(node.getExpression());
		statements.add(node);
		return super.visit(node);
	}

	/*
	 * (non-Javadoc) Traverses all the FOR statement nodes.
	 * 
	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.
	 * ForStatement)
	 */
	@Override
	public boolean visit(ForStatement node) {
		expressions.add(node.getExpression());
		statements.add(node);
		return super.visit(node);
	}

	/*
	 * (non-Javadoc) Traverses all the WHILE statement nodes.
	 * 
	 * @see org.eclipse.jdt.core.dom.ASTVisitor#visit(org.eclipse.jdt.core.dom.
	 * WhileStatement)
	 */
	@Override
	public boolean visit(WhileStatement node) {
		expressions.add(node.getExpression());
		statements.add(node);
		return super.visit(node);
	}

	/**
	 * @return the list of expression nodes
	 */
	public List<Expression> getExpressions() {
		return expressions;
	}

	/**
	 * @return the list of all statements.
	 */
	public List<Statement> getStatements() {
		return statements;
	}

}
