package edu.stanford.nlp.hcoref.sieve;

import java.util.List;
import java.util.Properties;

import edu.stanford.nlp.hcoref.data.Dictionaries;
import edu.stanford.nlp.hcoref.data.Document;
import edu.stanford.nlp.hcoref.data.Mention;
import edu.stanford.nlp.hcoref.data.Dictionaries.MentionType;

public class OracleSieve extends Sieve {

  private static final long serialVersionUID = 3510248899162246138L;

  public OracleSieve(Properties props, String sievename) {
    super(props, sievename);
    this.classifierType = ClassifierType.ORACLE;
  }

  @Override
  public void findCoreferentAntecedent(Mention m, int mIdx, Document document, Dictionaries dict, Properties props, StringBuilder sbLog) throws Exception {
    for(int distance=0 ; distance <= m.sentNum ; distance++) {
      List<Mention> candidates = document.predictedMentions.get(m.sentNum-distance);

      for(Mention candidate : candidates) {
        if(!matchedMentionType(candidate, aTypeStr) || !matchedMentionType(m, mTypeStr)) continue;
//        if(!options.mType.contains(m.mentionType) || !options.aType.contains(candidate.mentionType)) continue;
        if(candidate == m) continue;
        if(distance==0 && m.appearEarlierThan(candidate)) continue;   // ignore cataphora

        if(Sieve.isReallyCoref(document, m.mentionID, candidate.mentionID)) {
          if(m.mentionType==MentionType.LIST) {
            System.err.println("LIST MATCHING MENTION : "+m.spanToString()+"\tANT: "+candidate.spanToString());
          }
          Sieve.merge(document, m.mentionID, candidate.mentionID);
          return;
        }   
      }   
    }
  }
}
